#include "functions.h"

void initDevice()
{
    // osc setup
    OSCCON = 0x70; // 16MHz
    OSCCON2 = 0x00; // power off primary, secondarry osc, use only hfintosc
    while (!OSCCONbits.IOFS); // wait until osc is stable

    // disabling analog pins - only digital
    ANSELA = 0x00;
    ANSELB = 0x00;
    ANSELC = 0x00;

    //LCD I2C setup
    SSP1CON1 = 0x28; //MSSP enabled, clock stretch, I2C master Fosc/(4*(SSPxADD+1))
    SSP1ADD = 0x09; // 400kHz clock
    SSP1STAT = 0x00; // slew rate control
    TRISCbits.RC5 = 0;
    TRISC |= 0x18; // SDA, SCL as inputs
    LATCbits.LATC5 = 1; //~RST = 1 -> no reset

    // USART Debug interface setup
    TRISC |= 0xc0; //TX, RX usage
    SPBRG1 = 25; // 9600 Bauds/s, with 16MHz clk
    BAUDCON1 = 0x50;
    RCSTA1 = 0x90;// Async Receiver enabled
    PIE1bits.RC1IE = 1; // Receiver Interrupt
    INTCONbits.PEIE_GIEL = 1; // Peripheral interrupt
    INTCONbits.GIE_GIEH = 1; // global interrupt

    TXSTA1 = 0x20; // Transmit enabled
}

bool I2CStart()
{
    // start condition
    SSP1CON2bits.SEN = 1;
    while(SSP1CON2bits.SEN);
    // transmit address
    SSP1BUF = LCD_ADDR;
    if(SSP1CON2bits.ACKSTAT)
    {
        // ACK not received - error, send stop condition (P) and exit
        SSP1CON2bits.PEN = 1;
        return false;
    }
    return true;
}

bool I2CSend(unsigned char data)
{
    SSP1BUF = LCD_ADDR;
    if(SSP1CON2bits.ACKSTAT)
    {
        // ACK not received - error, send stop condition (P) and exit
        SSP1CON2bits.PEN = 1;
        return false;
    }
    SSP1BUF = 0x00;
    if(SSP1CON2bits.ACKSTAT)
    {
        // ACK not received - error, send stop condition (P) and exit
        SSP1CON2bits.PEN = 1;
        return false;
    }
    SSP1BUF = data;
    if(SSP1CON2bits.ACKSTAT)
    {
        // ACK not received - error, send stop condition (P) and exit
        SSP1CON2bits.PEN = 1;
        return false;
    }
    return true;
}

bool I2CWriteCommand(unsigned char cmd, bool cnt)
{
    unsigned char data = cmd;
    if(cnt)
        data |= 0x90;
    else
        data &= 0x7F;
    data |= 0x40; // command type
    // transmit data if not busy
    while(SSP1STATbits.R_nW);
    SSP1BUF = data;
    if(SSP1CON2bits.ACKSTAT)
    {
        // ACK not received
        SSP1CON2bits.PEN = 1;
        return false;
    }
    return true;
}

bool I2CWriteRamData(unsigned char cmd, bool cnt)
{
    unsigned char data = cmd;
   if(cnt)
        data |= 0x90;
    else
        data &= 0x7F;
    data &= 0xCF; // RAM type
    // transmit data
    while(SSP1STATbits.R_nW);
    SSP1BUF = data;
    if(SSP1CON2bits.ACKSTAT)
    {
        // ACK not received
        SSP1CON2bits.PEN = 1;
        return false;
    }
    return true;
}

bool I2CWriteData(unsigned char data)
{
    // transmit data
    while(SSP1STATbits.R_nW);
    SSP1BUF = data;
    if(SSP1CON2bits.ACKSTAT)
    {
        // ACK not received
        SSP1CON2bits.PEN = 1;
        return false;
    }
    return true;
}

bool I2CStop()
{
    while(SSP1STATbits.R_nW);
    SSP1CON2bits.PEN = 1;
    return true;
}

bool UARTWrite(unsigned char byte)
{
    while(!TXSTA1bits.TRMT);
    TXREG1 = byte;
    return true;
}
unsigned char UARTRead()
{
    return TXSTA1bits.TRMT;
}

bool UARTSendString(char*text)
{
    int i = 0;
    while(text[i]!= NULL)
    {
        UARTWrite(text[++i]);
    }
    return true;
}